# FILE: makerules.tcl
#
# This file controls the application 'pimake' by defining rules which
# describe how to build the applications and/or extensions produced by
# the source code in this directory.
#
# Verify that this script is being sourced by pimake
if {[llength [info commands MakeRule]] == 0} {
    error "'[info script]' must be evaluated by pimake"
}
#
########################################################################

set mmdispObjects {
    mmdispcmds
    bitmap
    colormap
    display
}

set allObjects [concat avf2ovf avf2ppm avf2vio avf2ave mmdispsh $mmdispObjects]

MakeRule Define {
    -targets            all
    -dependencies       [concat configure \
	        [Platform Executables {avf2ovf avf2ppm avf2vio avf2ave} ] \
                [Platform Specific appindex.tcl]]
}

MakeRule Define {
    -targets            configure
    -dependencies       [Platform Name]
}

MakeRule Define {
    -targets            [Platform Name]
    -dependencies       {}
    -script             {MakeDirectory [Platform Name]}
}

MakeRule Define {
    -targets            tclIndex
    -dependencies       [glob -nocomplain *.tcl]
    -script             {
                         MakeTclIndex .
                        }
}

# Could use better support for this:
MakeRule Define {
    -targets            [Platform Specific appindex.tcl]
    -dependencies       [Platform Executables {mmdispsh avf2ave avf2ovf
                                               avf2ppm avf2vio} ]
    -script             {puts "Updating [Platform Specific appindex.tcl]"
                        set f [open [Platform Specific appindex.tcl] w]
                        puts $f [format {
Oc_Application Define {
    -name               mmDispSh
    -version            1.1.0.1
    -machine            %s
    -file               "%s"
}
Oc_Application Define {
    -name               avf2ave
    -version            1.1.0.1
    -machine            %s
    -file               "%s"
}
Oc_Application Define {
    -name               avf2ovf
    -version            1.1.0.1
    -machine            %s
    -file               "%s"
}
Oc_Application Define {
    -name               avf2ppm
    -version            1.1.0.1
    -machine            %s
    -file               "%s"
}
Oc_Application Define {
    -name               avf2vio
    -version            1.1.0.1
    -machine            %s
    -file               "%s"
}
                        } [Platform Name] \
				[file tail [Platform Executables mmdispsh]] \
			  [Platform Name] \
				[file tail [Platform Executables avf2ave]] \
			  [Platform Name] \
				[file tail [Platform Executables avf2ovf]] \
			  [Platform Name] \
				[file tail [Platform Executables avf2ppm]] \
			  [Platform Name] \
				[file tail [Platform Executables avf2vio]]]
                        close $f
                        }
}

MakeRule Define {
    -targets            [Platform Executables mmdispsh]
    -dependencies       [concat [Platform Objects $mmdispObjects] \
				[Platform Objects mmdispsh] \
			        [Platform StaticLibraries {vf nb oc}] \
			        [list [file join .. .. ext oc tclIndex] \
			        [file join .. .. ext ow tclIndex] \
			        [file join .. .. ext net tclIndex] \
			        [file join .. .. ext vf tclIndex] \
			        [file join .. .. build [Platform Name]] \
			        tclIndex]]
    -script		[format {
			    Platform Link -obj {mmdispsh %s} \
			            -lib {vf nb oc tk} \
			            -sub WINDOWS -out mmdispsh
			} $mmdispObjects]
}

MakeRule Define {
    -targets            [Platform Executables avf2ovf]
    -dependencies       [concat [Platform Objects avf2ovf] \
			        [Platform StaticLibraries {vf nb oc} ] \
			        [list [file join .. .. build [Platform Name]]]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {vf nb oc tk} \
			            -sub WINDOWS -out avf2ovf
			} avf2ovf]
}

set avf2ppmObjects {
    avf2ppm
    bitmap
    colormap
    display
}

MakeRule Define {
    -targets            [Platform Executables avf2ppm]
    -dependencies       [concat [Platform Objects  $avf2ppmObjects] \
			        [Platform StaticLibraries {vf nb oc} ]          \
			        [list [file join .. .. build [Platform Name]]]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {vf nb oc tk} \
			            -sub WINDOWS -out avf2ppm
			} $avf2ppmObjects ]
}

MakeRule Define {
    -targets            [Platform Executables avf2vio]
    -dependencies       [concat [Platform Objects avf2vio] \
			        [Platform StaticLibraries {vf nb oc} ] \
			        [list [file join .. .. build [Platform Name]]]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {vf nb oc tk} \
			            -sub WINDOWS -out avf2vio
			} avf2vio]
}

MakeRule Define {
    -targets            [Platform Executables avf2ave]
    -dependencies       [concat [Platform Objects avf2ave] \
			        [Platform StaticLibraries {vf nb oc} ] \
			        [list [file join .. .. build [Platform Name]]]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {vf nb oc tk} \
			            -sub WINDOWS -out avf2ave
			} avf2ave]
}

foreach o $allObjects { 
    MakeRule Define {
        -targets	[Platform Objects $o]
        -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ $o.cc] Dependencies]]
        -script		[format {Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ %s.cc] DepPath] \
			        -src %s.cc -out %s
			} $o $o $o]
    }
}

MakeRule Define {
    -targets		upgrade
    -script		{DeleteFiles avftoave.cc avftoovf.cc avftovio.cc
			 DeleteFiles mmdinit.cc mmdinit.h mmdisp.cc
			 DeleteFiles mmdisp.h mmdisp.tcl
			}
}

MakeRule Define {
    -targets            distclean
    -dependencies       clean
    -script             {DeleteFiles [Platform Name]}
}
 
MakeRule Define {
    -targets            clean
    -dependencies       mostlyclean
}
 
MakeRule Define {
    -targets            mostlyclean
    -dependencies       objclean
    -script             {eval DeleteFiles [concat tclIndex \
	        [Platform Specific appindex.tcl] [Platform \
                Executables {avf2ppm avf2ovf avf2vio avf2ave mmdisp mmdispsh} ]]}
}
 
MakeRule Define {
    -targets            objclean
    -dependencies       {}
    -script             [format {
			    eval DeleteFiles [Platform Objects {%s} ]
			} $allObjects]
}

unset allObjects
unset mmdispObjects
